@@ -1,7 +1,7 @@
 # Python implementation of the MySQL client-server protocol
 # http://dev.mysql.com/doc/internals/en/client-server-protocol.html
 # Error codes:
-# http://dev.mysql.com/doc/refman/5.5/en/error-messages-client.html
+# https://dev.mysql.com/doc/refman/5.5/en/error-handling.html
 from __future__ import print_function
 from ._compat import  JYTHON, IRONPYTHON
 
@@ -29,6 +29,8 @@ from .protocol import (
 from .util import byte2int, int2byte
 from . import err, VERSION_STRING
 
+BUFLEN=32768
+
 try:
     import ssl
     SSL_ENABLED = True
@@ -116,15 +118,13 @@ class Connection(object):
         the interface from which to connect to the host. Argument can be
         a hostname or an IP address.
     :param unix_socket: Optionally, you can use a unix socket rather than TCP/IP.
-    :param read_timeout: The timeout for reading from the connection in seconds (default: None - no timeout)
-    :param write_timeout: The timeout for writing to the connection in seconds (default: None - no timeout)
     :param charset: Charset you want to use.
     :param sql_mode: Default SQL_MODE to use.
     :param read_default_file:
         Specifies  my.cnf file to read these parameters from under the [client] section.
     :param conv:
         Conversion dictionary to use instead of the default one.
-        This is used to provide custom marshalling and unmarshaling of types.
+        This is used to provide custom marshalling and unmarshalling of types.
         See converters.
     :param use_unicode:
         Whether or not to default to unicode strings.
@@ -148,7 +148,7 @@ class Connection(object):
         The class needs an async ``authenticate`` method taking an authentication packet as
         an argument.  For the dialog plugin, a prompt(echo, prompt) method can be used
         (if no authenticate method) for returning a string from the user. (experimental)
-    :param server_public_key: SHA256 authenticaiton plugin public key value. (default: None)
+    :param server_public_key: SHA256 authentication plugin public key value. (default: None)
     :param db: Alias for database. (for compatibility to MySQLdb)
     :param passwd: Alias for password. (for compatibility to MySQLdb)
     :param binary_prefix: Add _binary prefix on bytes and bytearray. (default: False)
@@ -179,9 +179,12 @@ class Connection(object):
                  compress=None, named_pipe=None,
                  autocommit=False, db=None, passwd=None, local_infile=False,
                  max_allowed_packet=16*1024*1024, defer_connect=False,
-                 auth_plugin_map={}, read_timeout=None, write_timeout=None,
+                 auth_plugin_map={},
                  bind_address=None, binary_prefix=False, program_name=None,
                  server_public_key=None):
+        self._rbuf = bytearray()
+        self._rbuf_offset = 0
+
         if use_unicode is None and sys.version_info[0] > 2:
             use_unicode = True
 
@@ -244,6 +247,8 @@ class Connection(object):
 
         self.host = host or "localhost"
         self.port = port or 3306
+        if type(self.port) is not int:
+            raise ValueError("port should be of type int")
         self.user = user or DEFAULT_USER
         self.password = password or b""
         if isinstance(self.password, str):
@@ -254,12 +259,6 @@ class Connection(object):
         if not (0 < connect_timeout <= 31536000):
             raise ValueError("connect_timeout should be >0 and <=31536000")
         self.connect_timeout = connect_timeout or None
-        if read_timeout is not None and read_timeout <= 0:
-            raise ValueError("read_timeout should be >= 0")
-        self._read_timeout = read_timeout
-        if write_timeout is not None and write_timeout <= 0:
-            raise ValueError("write_timeout should be >= 0")
-        self._write_timeout = write_timeout
         if charset:
             self.charset = charset
             self.use_unicode = True
@@ -328,15 +327,6 @@ class Connection(object):
         ctx.options |= ssl.OP_NO_SSLv3
         return ctx
 
-    async def __aexit__(self, *tb):
-        if self._curs is not None:
-            c,self._curs = self._curs,None
-            return await c.__aexit__(*tb)
-
-        with trio.move_on_after(1) as scope:
-            scope.shield = True
-            await self.aclose()
-
     async def aclose(self):
         """
         Send the quit message and close the socket.
@@ -574,6 +564,15 @@ class Connection(object):
                 raise
         raise RuntimeError("You can't nest cursors")
 
+    async def __aexit__(self, *tb):
+        if self._curs is not None:
+            c,self._curs = self._curs,None
+            return await c.__aexit__(*tb)
+
+        with trio.move_on_after(1) as scope:
+            scope.shield = True
+            await self.aclose()
+
     async def connect(self, sock=None):
         self._closed = False
         try:
@@ -641,9 +640,9 @@ class Connection(object):
 
     async def write_packet(self, payload):
         """Writes an entire "mysql packet" in its entirety to the network
-        addings its length and sequence number.
+        adding its length and sequence number.
         """
-        # Internal note: when you build packet manualy and calls _write_bytes()
+        # Internal note: when you build packet manually and calls _write_bytes()
         # directly, you should set self._next_seq_id properly.
         data = pack_int24(len(payload)) + int2byte(self._next_seq_id) + payload
         if DEBUG: dump_packet(data)
@@ -657,7 +656,7 @@ class Connection(object):
         :raise OperationalError: If the connection to the MySQL server is lost.
         :raise InternalError: If the packet sequence number is wrong.
         """
-        buff = b''
+        buff = bytearray()
         while True:
             packet_header = await self._read_bytes(4)
             #if DEBUG: dump_packet(packet_header)
@@ -685,15 +684,27 @@ class Connection(object):
             if bytes_to_read < MAX_PACKET_LEN:
                 break
 
-        packet = packet_type(buff, self.encoding)
-        packet.check_error()
+        packet = packet_type(bytes(buff), self.encoding)
+        if packet.is_error_packet():
+            if self._result is not None and self._result.unbuffered_active is True:
+                self._result.unbuffered_active = False
+            packet.raise_for_error()
         return packet
 
     async def _read_bytes(self, num_bytes):
-        rdata = b""
-        while self._sock is not None and len(rdata) < num_bytes:
+        rbuf = self._rbuf
+        if len(rbuf)-self._rbuf_offset >= num_bytes:
+            rdata = bytes(rbuf[self._rbuf_offset:self._rbuf_offset+num_bytes])
+            self._rbuf_offset += num_bytes
+            return rdata
+
+        del rbuf[:self._rbuf_offset]
+        self._rbuf_offset = 0
+
+        while self._sock is not None and len(rbuf) < num_bytes:
+            missing = num_bytes-len(rbuf)
             try:
-                data = await self._sock.receive_some(num_bytes-len(rdata))
+                data = await self._sock.receive_some(max(missing,BUFLEN))
             except trio.BrokenResourceError as e:
                 self._force_close()
                 raise err.OperationalError(
@@ -706,11 +717,13 @@ class Connection(object):
             else:
                 if data == b"":
                     break
-                rdata += data
-        if len(rdata) < num_bytes:
+                rbuf += data
+        if len(rbuf) < num_bytes:
             self._force_close()
             raise err.OperationalError(
                 CR.CR_SERVER_LOST, "Lost connection to MySQL server during query")
+        rdata = bytes(rbuf[:num_bytes])
+        self._rbuf_offset = num_bytes
         return rdata
 
     async def _write_bytes(self, data):
@@ -805,7 +818,7 @@ class Connection(object):
         if self.ssl and self.server_capabilities & CLIENT.SSL:
             await self.write_packet(data_init)
 
-            self._sock = trio.SSLStream(self._sock, self.ctx, server_hostname=self.host)
+            self._sock = trio.ssl.SSLStream(self._sock, self.ctx, server_hostname=self.host)
             await self._sock.do_handshake()
             self._secure = True
 
@@ -902,11 +915,13 @@ class Connection(object):
                     raise err.OperationalError(2059, "Authentication plugin '%s'"
                               " not loaded: - %r missing authenticate method" % (plugin_name, type(handler)))
         if plugin_name == b"caching_sha2_password":
-            return await _auth.caching_sha2_password_auth(self, auth_packet)
+            return _auth.caching_sha2_password_auth(self, auth_packet)
         elif plugin_name == b"sha256_password":
-            return await _auth.sha256_password_auth(self, auth_packet)
+            return _auth.sha256_password_auth(self, auth_packet)
         elif plugin_name == b"mysql_native_password":
             data = _auth.scramble_native_password(self.password, auth_packet.read_all())
+        elif plugin_name == b'client_ed25519':
+            data = _auth.ed25519_password(self.password, auth_packet.read_all())
         elif plugin_name == b"mysql_old_password":
             data = _auth.scramble_old_password(self.password, auth_packet.read_all()) + b'\0'
         elif plugin_name == b"mysql_clear_password":
@@ -1265,7 +1280,7 @@ class LoadLocalFile(object):
     async def send_data(self):
         """Send data packets from the local file to the server"""
         if not self.connection._sock:
-            raise err.InterfaceError("(0, '')")
+            raise err.InterfaceError(0, '')
         conn = self.connection
 
         try:
