@@ -9,7 +9,7 @@ from . import err
 
 
 #: Regular expression for :meth:`Cursor.executemany`.
-#: executemany only suports simple bulk insert.
+#: executemany only supports simple bulk insert.
 #: You can use it to load large dataset.
 RE_INSERT_VALUES = re.compile(
     r"\s*((?:INSERT|REPLACE)\b.+\bVALUES?\s*)" +
@@ -34,7 +34,6 @@ class Cursor(object):
     #: Max size of allowed statement is max_allowed_packet - packet_header_size.
     #: Default value of max_allowed_packet is 1048576.
     max_stmt_length = 1024000
-
     _defer_warnings = False
 
     def __init__(self, connection):
@@ -368,10 +367,10 @@ class Cursor(object):
         self.lastrowid = result.insert_id
         self._rows = result.rows
         self._warnings_handled = False
-
+ 
         if not self._defer_warnings:
             await self._show_warnings()
-
+ 
     async def _show_warnings(self):
         if self._warnings_handled:
             return
@@ -385,6 +384,9 @@ class Cursor(object):
             msg = w[-1]
             warnings.warn(err.Warning(*w[1:3]), stacklevel=4)
 
+    def __iter__(self):
+        return iter(self.fetchone, None)
+
     Warning = err.Warning
     Error = err.Error
     InterfaceError = err.InterfaceError
@@ -441,8 +443,6 @@ class SSCursor(Cursor):
     possible to scroll backwards, as only the current row is held in memory.
     """
 
-    _defer_warnings = True
-
     def _conv_row(self, row):
         return row
 
